<?php
/**
 * Callbacks Intertitres hierarchiques
 *
 * @plugin     Intertitres hierarchiques
 * @copyright  2016
 * @author     Mist. GraphX
 * @licence    GNU/GPL
 * @package    SPIP\Porte_plume_intertitres\wheels
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Callback de la wheel intertitre qui renvoie le tableau d'extraction de la regexp sous la forme :
 * 0 - la chaine complète
 * 1 - le groupe ouvrant
 * 2 - le type|level - ou string/contenu si intertitre spip
 * 3 - le contenu
 * 4 - le groupe fermant
 * 5 - les extenders attributs, class, id ou indicateur de raz de la numérotation
 *
 * @see http://zone.spip.org/trac/spip-zone/browser/_plugins_/todo/trunk/wheels/todo.php
 * @see http://lumadis.be/regex/test_regex.php?id=2938
 *
 * @param array $t
 *
 * @return string
 */
function intertitres($t) {
	// Initialisation des variables de base : ne pas utiliser de static car on peut mettre des intertitres
	// dans d'autres champs que le texte.
	static $base_level = 0;
	static $type = '';
	static $last_node = array();

	// Extraction des extenders : classes, id, propriétés additionnelles et indicateur de raz
	// - on traite ce bloc de suite car l'extender {raz} permet de remttre à zéro les compteurs de numérotation
	if (isset($t[5])) {
		extract_attributes($t[5], $attributs);
		if (!empty($attributs['raz'])) {
			// On remet tous les compteurs à zéro
			$base_level = 0;
			$type = '';
			$last_node = array();
		}
	}

	// Ajuster le niveau de l'item
	if(!$base_level){
		// Récupérer le niveau de base d'après la global
		preg_match('/[h](\d)/s',$GLOBALS['debut_intertitre'], $matches) ;
		$base_level = $matches[1];
	}
	$level = $t[2] ? $base_level + strlen($t[2]) - 1 : $base_level;

	// Déterminer le type de classe des titres : h ou r
	if (!$type) {
		$type = substr($t[2], 0, 1) == '#' ? 'r' : 'h';
	}

	// Calcul de la classe minimale de la balise hx
	$css = $type . $level;

	// Extraction du texte du titre
	$title = $t[3];

	// Numérotation du texte si demandé
	if ($type == 'r') {
		// Il faut numéroter les titres en considérant qu'ils sont détectés dans l'ordre.
		$imbric_level = $t[2] ? strlen($t[2]) - 1 : 0;
		if (isset($last_node[$imbric_level])) {
			// On rajoute un nouveau noeud
			$node = array_slice($last_node, 0, $imbric_level + 1);
			$node[$imbric_level] = isset($node[$imbric_level])
				? $node[$imbric_level] + 1
				: 1;
		} else {
			// On rajoute d'un nouveau niveau
			$node = $last_node;
			$node[$imbric_level] = 1;
		}
		$title = format_title($title, $node);
		$last_node = $node;
	}

	// Traitements des extenders hors raz (class, id, attribut)
	$id = '';
	$properties = '';
	if (!empty($attributs['css'])) {
		$css .= $attributs['css'];
	}
	if (!empty($attributs['id'])) {
		$id = "id=\"{$attributs['id']}\"";
	}
	if(!empty($attributs['properties'])) {
		$properties = ' ' . $attributs['properties'];
	}

	// Calcul du HTML compilé pour le titre : on ne dépasse pas h6
	if($level < 7) {
		$html = "<h${level} ${id} class=\"${css}\"${properties}>" . $title . "</h$level>";
	}
	else {
		$html = "<div ${id} class=\"${css}\"${properties}>" . $title . "</div>";
	}

	return $html;
}

function format_title($title, $node) {

	$number = '';
	foreach ($node as $_key => $_value) {
		$number .= $number ? ".${_value}" : $_value;
	}
	$title = "${number} - ${title}";

	return $title;
}

/**
 * Détermine la liste des attributs additionnels à ajouter à la balise hx.
 * Pour permettre d'utiliser des numéros dans plusieurs champs d'un objet, on ajoute un extender `raz`
 * indiquant à la wheel qu'il faut remettre les compteurs à zéro.
 *
 * @param string $extenders chaine extraite correspondant à des extenders possibles
 * @param array  $attributs Tableau des attributs contenant systématiquement les index `css`, `id`, `properties`, `raz`
 *                          qui peuvent être vides.
 *
 * @return void
 */
function extract_attributes($extenders, &$attributs) {

	$extenders = preg_split('/[\s]+/', $extenders);
	foreach($extenders as $extender){
		// css
		if ($extender === 'raz') {
			$attributs['raz'] = true;
		}
		elseif ($class = get_css($extender)) {
			$attributs['css'] .= ' ' . $class[1];
		}
		// id (un seul possible)
		elseif ($id = get_id($extender)) {
			$attributs['id'] = $id[1];
		}
		// properties
		elseif ($propertie = get_propertie($extender)) {
			$attributs['properties'] .= $propertie[1];
		}
	}
}

/**
 * Renvoie les classes additionnelles de la balise hx ou vide sinon.
 *
 * @see http://lumadis.be/regex/test_regex.php?id=2936
 *
 * @param string $str
 *
 * @return string
 */
function get_css($str) {
	$regex = '/\.([_a-zA-Z0-9-]+)/s';
	if(preg_match($regex,$str,$class_name))
		return $class_name;
	else
		return '';
}

/**
 * Renvoie l'id de la balise hx ou vide sinon.
 *
 * @param string $str
 *
 * @return string
 */
function get_id($str) {
	$regex = '/#([_a-zA-Z0-9-]+)/s';
	if(preg_match($regex,$str,$id))
		return $id;
	else
		return '';
}

/**
 * Renvoie les propriétés additionnelles de la balise hx ou vide sinon.
 *
 * @see http://lumadis.be/regex/test_regex.php?id=2937
 *
 * @param string $str
 *
 * @return string
 */
function get_propertie($str) {
	$regex = '/([_a-z-]+=[\"|\'].*?[\"|\'])/s';
	if(preg_match($regex,$str,$propertie))
		return $propertie;
	else
		return '';
}
